/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"
#include "SceneSerializer.h"

Light* blueLight;

// Listener class for frame updates
class PRMDemoFrameListener : public ExampleFrameListener
{
protected:
	
	unsigned long framecount;
	SceneNode* mainLightNode;
	Light* light;
	
public:
    PRMDemoFrameListener(RenderWindow* window, Camera* maincam, SceneNode* lightNode, Light* l)
        :ExampleFrameListener(window, maincam) 
				 
    {
		mainLightNode = lightNode;
		light = l;
		framecount = 0;	
		mMoveSpeed /= 50.0;
    }

    bool frameStarted(const FrameEvent& evt)
    {	
	/*	light->setDirection(mCamera->getDirection());
		light->setPosition(mCamera->getPosition());
		mainLightNode->setPosition(mCamera->getPosition());*/
		bool result = ExampleFrameListener::frameStarted(evt);
       framecount++;

	   static float t = 0;
	   float dt = evt.timeSinceLastFrame / 4.0;
	   t += dt;
/*
	   Vector3 pos(30 * sin(t), 1, 30 * cos(t));
	   light->getParentSceneNode()->setPosition(pos);
	   Vector3 dir = -pos;
	   dir.normalise();
	   light->setDirection(dir);*/
	  
	  
        return result;     
    }

	

};

/** Application class */
class PRMDemoApplication : public ExampleApplication
{




protected:
	SceneNode* kupolaNode;
	SceneNode* mainLightNode;
	Light* mainLight;
	
	void loadResources(void)
	{
		// Initialise, parse scripts etc
		ResourceGroupManager::getSingleton().initialiseResourceGroup("Bootstrap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("General");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Basic");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("PostProc");
		//ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_PathMap");
		//ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_PathMapDemo");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Demo_Towers");
	}

	void createPostproc()
	{		
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/Glow", true);
/*
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap", true);	*/
	}
    
	void chooseSceneManager()
	{
		mSceneMgr = mRoot->createSceneManager("OctreeSceneManager");
		//mSceneMgr = mRoot->createSceneManager("OcclusionCullingSceneManager");
	}

	/*
	void createScene(void) /// SHADOWACCUM
    {
		//std::string occlusion_cfg("OcclusionCulling.cfg");
		//mSceneMgr->setWorldGeometry(occlusion_cfg);		
		//createPostproc();

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		OgreIlluminationManager::getSingleton().setShadowMapMaterialName("GTP/Basic/CameraDepth");
		OgreIlluminationManager::getSingleton().setFocusingSM(false);
	   OgreIlluminationManager::getSingleton().setShadowMapSize(1024);
	   OgreIlluminationManager::getSingleton().setBlurShadowMap(false);
	   OgreIlluminationManager::getSingleton().setUseLISPSM(false);
	   OgreIlluminationManager::getSingleton().setAreaLigtRadius(2);
				
		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		mCamera->setPosition(0,0,0);
		//mCamera->setPosition(0,100,0);
		//mCamera->lookAt(0,-1,0);
		mCamera->setFOVy(Radian(Degree(80)));
		mCamera->setNearClipDistance(0.01);
		mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(0,0, 0));
		//mSceneMgr->setSkyBox(true, "Examples/EveningSkyBox", 500);
		
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

		Entity* pillar = mSceneMgr->createEntity("pillar", "pillarShape.mesh");
		SceneNode* pillarNode = rootNode->createChildSceneNode("pillarnode");
		pillarNode->attachObject(pillar);

			
		createPlane("p","GTP/Basic/ShadowReceiver/ShadowAccum",Vector3(0,-1.3,0),Vector2(20,20),Vector2(1,1));

		Light* mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_SPOTLIGHT);
		mainLight->setCastShadows(true);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		
		mainLightNode->setPosition(10,10,10);		
		mainLight->setPosition(10,10,10);
		mainLight->setPowerScale(10);
		mainLight->setDirection(-1,-1, -1);
		mainLight->setSpotlightOuterAngle(Radian(Degree(90)));
		mainLight->setAttenuation(20, 0, 0, 0);
		mainLightNode->attachObject(mainLight); 	

		BillboardSet* bbs = mSceneMgr->createBillboardSet("bb", 1);
		bbs->setDefaultDimensions(6,6);
		bbs->createBillboard(Vector3::ZERO, ColourValue::White);
		bbs->setMaterialName("Flare");
		mainLightNode->attachObject(bbs);
		
		OgreIlluminationManager::getSingleton().initTechniques();
   }*/

	void createScene(void) ///TOWERS
    {
		//std::string occlusion_cfg("OcclusionCulling.cfg");
		//mSceneMgr->setWorldGeometry(occlusion_cfg);
		
		//createPostproc();

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		OgreIlluminationManager::getSingleton().setShadowMapSize(700);
		OgreIlluminationManager::getSingleton().setShadowMapMaterialName("GTP/Basic/Depth");
		OgreIlluminationManager::getSingleton().setFocusingSM(true);
	   OgreIlluminationManager::getSingleton().setBlurShadowMap(true);
	   OgreIlluminationManager::getSingleton().setUseLISPSM(true);
				
		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		mCamera->setPosition(-10,20,-20);
		//mCamera->setPosition(0,100,0);
		mCamera->lookAt(10,0,20);
		mCamera->setFOVy(Radian(Degree(80)));
		mCamera->setNearClipDistance(0.01);
		mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(0.1, 0.1, 0.1));
		mSceneMgr->setSkyBox(true, "Examples/EveningSkyBox", 500);
		
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

		
		SceneSerializer s(mSceneMgr);
		DataStreamPtr inputStream; 				
		inputStream = ResourceGroupManager::getSingleton().openResource("towers.level");	//towers2.level	
		s.parseScript(inputStream, ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
			
		//createPlane("p","groundmaterial",Vector3(0,-0.3,0),Vector2(150,150));

		Light* mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_DIRECTIONAL);
		mainLight->setCastShadows(true);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		
		mainLightNode->setPosition(1000,1000,1000);		
		mainLight->setPosition(1000,1000,1000);
		mainLight->setPowerScale(5);
		mainLight->setDirection(-1,-1, -1);
		mainLightNode->attachObject(mainLight); 	
		
		OgreIlluminationManager::getSingleton().initTechniques();
   }
/*
	void createScene(void) ///PATHMAP Hangar
	{ 
		std::string occlusion_cfg("OcclusionCulling.cfg");
		//mSceneMgr->setWorldGeometry(occlusion_cfg);
		
		//createPostproc();

		OgreIlluminationManager::getSingleton().joinSharedRuns(false);
		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		OgreIlluminationManager::getSingleton().setShadowMapMaterialName(Light::LT_SPOTLIGHT, "GTP/Basic/Distance_Normalized");
		OgreIlluminationManager::getSingleton().setShadowMapMaterialName(Light::LT_POINT, "GTP/Basic/Distance_Normalized");
		OgreIlluminationManager::getSingleton().setFocusingSM(false);
	   OgreIlluminationManager::getSingleton().setShadowMapSize(512);
	   OgreIlluminationManager::getSingleton().setBlurShadowMap(false);
	   OgreIlluminationManager::getSingleton().setUseLISPSM(false);
				
		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		
		mCamera->setPosition(32,0.68,11);
		//mCamera->setPosition(50,35,-2);
		//mCamera->lookAt(0,0,0);
		
		mCamera->setFOVy(Radian(Degree(80)));
		mCamera->setNearClipDistance(0.01);
		mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(0.1, 0.1, 0.1));
		mSceneMgr->setSkyBox(true, "Examples/EveningSkyBox", 500);
		
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

		
		SceneSerializer s(mSceneMgr);
		DataStreamPtr inputStream; 				
		inputStream = ResourceGroupManager::getSingleton().openResource("processed.level");	//towers2.level	
		s.parseScript(inputStream, ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
		s.parseEntryPoints("SpacePrmEntryPoints.text");
			
		//createPlane("p","GTP/Basic/ShadowReceiver/Depth",Vector3(0,-0.3,0),Vector2(150,150));

		mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_SPOTLIGHT);
		mainLight->setCastShadows(true);
        mainLight->setDiffuseColour(ColourValue(1.0,0.5,0.3,1));
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		
		mainLightNode->setPosition(30,1,0.0);		
		//mainLight->setPosition(30,1,0);
		mainLight->setPowerScale(200);
		mainLight->setDirection(-1, 0, 0);
		mainLight->setSpotlightOuterAngle(Radian(Degree(90)));
		mainLight->setSpotlightFalloff(1);
		mainLight->setAttenuation(200, 0, 0, 1);
		mainLightNode->attachObject(mainLight); 	

		blueLight = mSceneMgr->createLight("BlueLight");
		blueLight->setType(Light::LT_SPOTLIGHT);
		blueLight->setCastShadows(true);
        blueLight->setDiffuseColour(ColourValue(0.3,0.5,1.0,1));
		blueLight->setSpecularColour(ColourValue::White);
		SceneNode* blueLightNode = rootNode->createChildSceneNode();
		
		blueLightNode->setPosition(-30,1,0.0);		
		//blueLight->setPosition(-30,1,0);
		blueLight->setPowerScale(200);
		blueLight->setDirection(1, 0, 0);
		blueLight->setSpotlightOuterAngle(Radian(Degree(160)));
		blueLight->setSpotlightFalloff(1);
		blueLight->setAttenuation(2000, 0, 0, 1);
		blueLightNode->attachObject(blueLight); 	

		
		OgreIlluminationManager::getSingleton().initTechniques();
   }
*/	
	void createPlane(Ogre::String _name, Ogre::String _material, Ogre::Vector3 _pos, Ogre::Vector2 _size, Ogre::Vector2 _subdivisions = Ogre::Vector2(1,1)) 
	{
		Plane _plane;
        _plane.normal = Vector3::UNIT_Y;
        _plane.d = 0;


        MeshManager::getSingleton().createPlane(_name + ".plane",
            ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
            _plane,_size.x,_size.y,1,1,true,1,_subdivisions.x,_subdivisions.y,Vector3::UNIT_Z);
        
		Entity *_entity = mSceneMgr->createEntity(_name + ".entity", _name + ".plane");
        _entity->setMaterialName(_material);

		SceneNode* _node = mSceneMgr->getRootSceneNode()->createChildSceneNode(_name + ".node");
		_node->attachObject(_entity);
		_node->setPosition(_pos);

	}

	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new PRMDemoFrameListener(mWindow,
											mCamera, mainLightNode, mainLight	);
       mFrameListener->setPriority(1);	
		mRoot->addFrameListener(mFrameListener);
		OgreIlluminationManager::getSingleton().setPriority(2);
		mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());
    }

};

